@@ -21,6 +21,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "channel_commands.h"
 
 #include "ssh.h"
@@ -33,8 +39,6 @@
 
 #include "channel_commands.c.x"
 
-#include <assert.h>
-
 void
 do_channel_open_command(struct command *s,
 			struct lsh_object *x,
@@ -56,6 +60,8 @@ do_channel_open_command(struct command *s,
 		      make_channel_open_exception(
 			SSH_OPEN_RESOURCE_SHORTAGE,
 			"Allocating a local channel number failed."));
+
+      return;
     }
 
   channel = NEW_CHANNEL(self, connection, index, &request);
