@@ -442,6 +442,8 @@ do_exc_userauth_disconnect(struct exception_handler *s,
       EXCEPTION_RAISE(self->connection->e,
 		      make_protocol_exception(SSH_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE,
 					      NULL));
+      /* Also raise the exception with the parent exception
+	 handler. */
     }
 
   EXCEPTION_RAISE(s->parent, e);
