@@ -23,6 +23,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
+#include "nettle/dsa.h"
+
 #include "publickey_crypto.h"
 
 #include "format.h"
@@ -30,10 +38,6 @@
 #include "sexp.h"
 #include "werror.h"
 
-#include "nettle/dsa.h"
-
-#include <assert.h>
-
 
 /* FIXME: Let caller supply the progress function. */
 
