@@ -33,9 +33,6 @@
 #include <unistd.h>
 #endif
 
-/* For the popen code */
-#include <wait.h>
-
 #ifdef HAVE_POLL
 # if HAVE_POLL_H
 #  include <poll.h>
@@ -52,6 +49,8 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
+/* For the popen code */
+#include <sys/wait.h>
 
 #include <arpa/inet.h>
 
@@ -72,7 +71,6 @@
 # define MY_POLLIN POLLIN
 #endif /* !POLLRDNORM */
 
-
 #define GABA_DEFINE
 #include "io.h.x"
 #undef GABA_DEFINE
@@ -714,7 +712,7 @@ do_listen_callback(struct io_callback *s,
 #if WITH_IPV6
   struct sockaddr_storage peer;
 #else
-  struct sockaddr peer;
+  struct sockaddr_in peer;
 #endif
 
   socklen_t addr_len = sizeof(peer);
@@ -788,7 +786,7 @@ do_connect_callback(struct io_callback *s,
     }
 }
 
-static struct io_callback *
+struct io_callback *
 make_connect_callback(struct command_continuation *c)
 {
   NEW(io_connect_callback, self);
@@ -1006,37 +1004,12 @@ int get_portno(const char *service, const char *protocol)
     }
 }
 
-
-/* If def != 0, use that value as a fallback if the lookup fails. */
-struct address_info *
-make_address_info_c(const char *host,
-		    const char *port,
-		    int def)
-{
-  int portno = get_portno(port, "tcp");
-  if (!portno)
-    portno = def;
-
-  if (!portno)
-    return NULL;
-
-  else
-    {
-      NEW(address_info, info);
-      
-      info->port = portno;
-      info->ip = host ? ssh_format("%lz", host) : NULL;
-      
-      return info;
-    }
-}
-
 struct address_info *
 make_address_info(struct lsh_string *host, uint32_t port)
 {
   NEW(address_info, info);
 
-  info->port = port; /* htons(port); */
+  info->port = port;
   info->ip = host;
   return info;
 }
@@ -1100,7 +1073,7 @@ fd2info(struct lsh_fd *fd, int side)
 #if WITH_IPV6
   struct sockaddr_storage sock;
 #else
-  struct sockaddr sock;
+  struct sockaddr_in sock;
 #endif
   
   socklen_t s_len = sizeof(sock);
@@ -1139,6 +1112,134 @@ choose_address(struct addrinfo *list,
 }
 #endif /* HAVE_GETADDRINFO */
 
+/* Uses NUL-terminated strings. HOST can be NULL, meaning any address.
+ * If PORT is NULL or can't be looked up, and DEFAULT_PORT is non-zero,
+ * the DEFAULT_PORT is used. */
+unsigned
+io_resolv_address(const char *host, const char *service,
+		  unsigned default_port,
+		  struct addr_queue *q)
+{
+  unsigned naddresses = 0;
+  
+  /* Some systems have getaddrinfo, but still doesn't implement all of
+   * RFC 2553 */
+#if defined(HAVE_GETADDRINFO) && \
+    defined(HAVE_GAI_STRERROR) && defined(HAVE_AI_NUMERICHOST)
+  {
+    struct addrinfo hints;
+    struct addrinfo *list;
+    struct addrinfo *p;
+    int err;
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_PASSIVE;
+
+    if (!service || (err = getaddrinfo(host, service, &hints, &list)))
+      {
+	/* FIXME: It seems ugly to have to convert the port number to
+	 * a string. Perhaps we can call getaddrinfo with a NULL or
+	 * dummy service and patch in the right port number later? */
+	
+	struct lsh_string *port = ssh_format("%di", default_port);
+	err = getaddrinfo(host, lsh_get_cstring(port), &hints, &list);
+	lsh_string_free(port);
+      }
+    
+    if (err)
+      {
+	debug("io_listen_address: getaddrinfo failed (err = %i): %z\n",
+	      err, gai_strerror(err));
+	return 0;
+      }
+
+    for (p = list; p; p = p->ai_next)
+      {
+	/* FIXME: Do we need to filter out some address families? */
+	struct sockaddr *n = addr_queue_add_tail(q, p->ai_addrlen);
+	memcpy(n, p->ai_addr, p->ai_addrlen);
+	naddresses++;
+      }
+    
+    freeaddrinfo(list);
+  }
+#else
+  /* !(defined(HAVE_GETADDRINFO) &&
+     defined(HAVE_GAI_STRERROR) && defined(HAVE_AI_NUMERICHOST) */ 
+
+#if WITH_IPV6
+#error IPv6 enabled, but getaddrinfo and friends were not found. 
+#endif
+
+  if (strchr(host, ':'))
+    {
+      debug("io_resolv_address: Literal IPv6 used. Failing.\n");
+      return 0;
+    }
+  else
+    {
+      struct sockaddr_in addr;
+      unsigned port;
+      if (!service || !(port = get_portno(service, "tcp")))
+	port = default_port;
+
+      /* Needed to clear addr.sin_zero on systems that have that
+       * field. In particular, AIX. */
+      memset(&addr, 0, sizeof(addr));
+      
+      addr.sin_port = htons(port);
+
+      /* Use IPv4 only */
+      addr.sin_family = AF_INET;
+    
+      if (!host)
+	/* Any interface */
+	addr.sin_addr.s_addr = INADDR_ANY;
+
+      else
+	{
+	  /* First check for numerical ip-number */
+#if HAVE_INET_ATON
+	  if (!inet_aton(host, &addr.sin_addr))
+#else /* !HAVE_INET_ATON */
+	    /* NOTE: It is wrong to work with ((unsigned long int) -1)
+	     * directly, as this breaks Linux/Alpha systems. But
+	     * INADDR_NONE isn't portable. That's what inet_aton is for;
+	     * see the GNU libc documentation. */
+# ifndef INADDR_NONE
+# define INADDR_NONE ((unsigned long int) -1)
+# endif /* !INADDR_NONE */
+	  addr.sin_addr.s_addr = inet_addr(host);
+	  if (addr.sin_addr.s_addr == INADDR_NONE)
+#endif  /* !HAVE_INET_ATON */
+	    {
+	      struct hostent *hp;
+	      unsigned i;
+	      if (! ( (hp = gethostbyname(host))
+		      && (hp->h_addrtype == AF_INET)))
+		return 0;
+
+	      assert(hp->h_length == sizeof(addr.sin_addr));
+	      
+	      for (i = 0; hp->h_addr_list[i]; i++)
+		{
+		  struct sockaddr *n;
+		  memcpy(&addr.sin_addr, hp->h_addr_list[i], hp->h_length);
+		  n = addr_queue_add_tail(q, sizeof(addr));
+		  memcpy(n, &addr, sizeof(addr));
+		  naddresses++;
+		}
+	    }
+	}
+    }
+#endif /* !HAVE_GETADDRINFO */
+    
+  return naddresses;
+}
+
+
 /* FIXME: Perhaps this function should be changed to return a list of
  * sockaddr:s? */
 struct sockaddr *
@@ -1280,16 +1381,30 @@ address_info2sockaddr(socklen_t *length,
 #endif /* !HAVE_GETADDRINFO */  
 }
 
+static void
+handle_nonblock_error(const char *msg)
+{
+  /* On BSD, trying to set /dev/null in nonblocking mode fails with
+   * errno 19, ENODEV. We have to ignore that.
+   *
+   * For now, still display a warning message, to keep track if when
+   * and where it occurs.
+   */
+  if (errno == ENODEV)
+    werror("%z %e\n", msg, errno);
+  else
+    fatal("%z %e\n", msg, errno);
+}
 
 void io_set_nonblocking(int fd)
 {
   int old = fcntl(fd, F_GETFL);
 
   if (old < 0)
-    fatal("io_set_nonblocking: fcntl(F_GETFL) failed %e", errno);
+    fatal("io_set_nonblocking: fcntl(F_GETFL) failed %e\n", errno);
   
   if (fcntl(fd, F_SETFL, old | O_NONBLOCK) < 0)
-    fatal("io_set_nonblocking: fcntl(F_SETFL) failed %e", errno);
+    handle_nonblock_error("io_set_nonblocking: fcntl(F_SETFL) failed");
 }
 
 void io_set_blocking(int fd)
@@ -1297,10 +1412,10 @@ void io_set_blocking(int fd)
   int old = fcntl(fd, F_GETFL);
 
   if (old < 0)
-    fatal("io_set_blocking: fcntl(F_GETFL) failed %e", errno);
+    fatal("io_set_blocking: fcntl(F_GETFL) failed %e\n", errno);
   
   if (fcntl(fd, F_SETFL, old & ~O_NONBLOCK) < 0)
-    fatal("io_set_blocking: fcntl(F_SETFL) failed %e", errno);
+    handle_nonblock_error("io_set_blocking: fcntl(F_SETFL) failed");
 }
 
 void io_set_close_on_exec(int fd)
@@ -1311,7 +1426,7 @@ void io_set_close_on_exec(int fd)
   int old = fcntl(fd, F_GETFD);
 
   if (old < 0)
-    fatal("io_set_nonblocking: fcntl(F_GETFD) failed %e", errno);
+    fatal("io_set_close_on_exec: fcntl(F_GETFD) failed %e\n", errno);
   
   if (fcntl(fd, F_SETFD, old | 1) < 0)
     fatal("Can't set close-on-exec flag for fd %i %e\n", fd, errno);
@@ -1374,7 +1489,7 @@ io_nfiles(void)
 struct lsh_fd *
 io_connect(struct sockaddr *remote,
 	   socklen_t remote_length,
-	   struct command_continuation *c,
+	   struct io_callback *c,
 	   struct exception_handler *e)
 {
   int s = socket(remote->sa_family, SOCK_STREAM, 0);
@@ -1408,12 +1523,166 @@ io_connect(struct sockaddr *remote,
 
   fd = make_lsh_fd(s, "connecting socket", e);
   
-  fd->write = make_connect_callback(c);
+  fd->write = c;
   lsh_oop_register_write_fd(fd);
     
   return fd;
 }
 
+/* connect_list_state */
+static void
+do_connect_list_kill(struct resource *s)
+{
+  CAST(connect_list_state, self, s);
+
+  if (self->super.alive)
+    {
+      unsigned i;
+      self->super.alive = 0;
+      for (i = 0; i < CONNECT_ATTEMPTS_LIMIT; i++)
+	{
+	  struct lsh_fd *fd = self->fds[i];
+	  if (fd)
+	    KILL_RESOURCE(&fd->super);
+	}
+    }
+}
+
+struct connect_list_state *
+make_connect_list_state(void)
+{
+  NEW(connect_list_state, self);
+  unsigned i;
+
+  init_resource(&self->super, do_connect_list_kill);
+  addr_queue_init(&self->q);
+  self->nfds = 0;
+  
+  for (i = 0; i < CONNECT_ATTEMPTS_LIMIT; i++)
+    self->fds[i] = NULL;
+
+  return self;
+}
+
+/* GABA:
+   (class
+     (name connect_list_callback)
+     (super io_callback)
+     (vars
+       (state object connect_list_state)
+       (index . unsigned)
+       (c object command_continuation)
+       (e object exception_handler)))
+*/
+
+static void
+connect_attempt(struct connect_list_callback *self)
+{
+  struct connect_list_state *state = self->state;
+  struct lsh_fd * fd;
+  socklen_t addr_length;
+  struct sockaddr *addr;
+  
+  assert(state->super.alive);
+
+  state->fds[self->index] = NULL;
+  
+  while ( (addr = addr_queue_peek_head(&state->q, &addr_length)))
+    {
+      fd = io_connect(addr, addr_length,
+		      &self->super, self->e);
+      addr_queue_remove_head(&state->q);
+
+      if (fd)
+	{
+	  state->fds[self->index] = fd;
+	  state->nfds++;
+	  break;
+	}
+    }
+}
+
+static void
+do_connect_list_callback(struct io_callback *s,
+			 struct lsh_fd *fd)
+{
+  CAST(connect_list_callback, self, s);
+  int socket_error;
+  socklen_t len = sizeof(socket_error);
+  
+  assert(self->state->nfds);
+  assert(self->index < CONNECT_ATTEMPTS_LIMIT);
+  assert(fd == self->state->fds[self->index]);
+
+  /* Check if the connection was successful */
+  if ((getsockopt(fd->fd, SOL_SOCKET, SO_ERROR,
+		  (char *) &socket_error, &len) < 0)
+      || socket_error)
+    {
+      trace("io.c: connect_callback: Connect on fd %i failed.\n", fd->fd);
+      close_fd(fd);
+      self->state->nfds--;
+
+      connect_attempt(self);
+      if (!self->state->nfds)
+	{
+	  /* All addresses failed */
+	  EXCEPTION_RAISE(self->e,
+			  make_io_exception(EXC_IO_CONNECT, NULL,
+					    errno, NULL));
+	  KILL_RESOURCE(&self->state->super);
+	  return;
+	}
+    }
+  else
+    { /* Success */
+      /* Remove the successful fd, and kill the rest */
+      self->state->fds[self->index] = NULL;
+      KILL_RESOURCE(&self->state->super);
+      fd->label = "connected socket";
+      COMMAND_RETURN(self->c, fd);
+    }
+}
+
+static struct connect_list_callback *
+make_connect_list_callback(struct connect_list_state *state,
+			   unsigned index,
+			   struct command_continuation *c,
+			   struct exception_handler *e)
+{
+  NEW(connect_list_callback, self);
+
+  self->super.f = do_connect_list_callback;
+  self->state = state;
+  self->index = index;
+  self->c = c;
+  self->e = e;
+  
+  return self;
+}
+
+/* Tries to connect to the addresses in the list, by attempting
+ * CONNECT_ATTEMPTS_LIMIT connect calls in parallel. The first
+ * successful connection (if any) is returned.
+ *
+ * Addresses are removed from the list as we go. */
+struct resource *
+io_connect_list(struct connect_list_state *state,
+		struct command_continuation *c,
+		struct exception_handler *e)
+{
+  unsigned i;
+
+  for (i = 0;
+       !addr_queue_is_empty(&state->q)
+	 && i < CONNECT_ATTEMPTS_LIMIT;
+       i++)
+    connect_attempt(make_connect_list_callback(state, i, c, e));
+
+  return &state->super;
+}
+
+
 struct lsh_fd *
 io_bind_sockaddr(struct sockaddr *local,
 		 socklen_t length,
@@ -1432,7 +1701,7 @@ io_bind_sockaddr(struct sockaddr *local,
     setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char*)&yes, sizeof yes);
   }
 
-  if (bind(s, (struct sockaddr *)local, length) < 0)
+  if (bind(s, local, length) < 0)
     {
       trace("io.c: bind failed %e\n", errno);
       close(s);
@@ -1442,7 +1711,6 @@ io_bind_sockaddr(struct sockaddr *local,
   return make_lsh_fd(s, "bound socket", e);
 }
 
-/* FIXME: Rename to io_listen. */
 struct lsh_fd *
 io_listen(struct lsh_fd *fd,
 	  struct io_callback *callback)
@@ -1466,6 +1734,45 @@ io_listen(struct lsh_fd *fd,
   return fd;
 }
 
+struct resource *
+io_listen_list(struct addr_queue *addresses,
+	       struct io_callback *callback,
+	       struct exception_handler *e)
+{
+  struct resource_list *resources = make_resource_list();
+  unsigned nbound = 0;
+  socklen_t addr_length;
+  struct sockaddr *addr;
+  
+  for (nbound = 0;
+       (addr = addr_queue_peek_head(addresses, &addr_length));
+       addr_queue_remove_head(addresses))
+    {
+      struct lsh_fd *fd;
+      debug("listen_list: Trying to bind address of type %i\n",
+	    addr->sa_family);
+
+      fd = io_bind_sockaddr(addr, addr_length, e);
+      if (fd)
+	{
+	  if (io_listen(fd, callback))
+	    {
+	      remember_resource(resources, &fd->super);
+	      nbound++;
+	    }
+	  else
+	    close_fd(fd);
+	}
+    }
+  if (nbound)
+    return &resources->super;
+  else
+    {
+      KILL_RESOURCE_LIST(resources);
+      return NULL;
+    }
+}
+
 
 /* AF_LOCAL sockets */
 
@@ -1738,7 +2045,8 @@ io_connect_local(struct local_info *info,
   if (old_cd < 0)
     return NULL;
   
-  fd = io_connect( (struct sockaddr *) addr, addr_length, c, e);
+  fd = io_connect( (struct sockaddr *) addr, addr_length,
+		   make_connect_callback(c), e);
 
   lsh_popd(old_cd, cdir);
 
@@ -1818,17 +2126,6 @@ io_write_file(const char *fname, int flags, int mode,
 		  block_size, c);
 }
 
-struct lsh_fd *
-io_read_file(const char *fname, 
-	     struct exception_handler *e)
-{
-  int fd = open(fname, O_RDONLY);
-  if (fd < 0)
-    return NULL;
-
-  return make_lsh_fd(fd, "read-only file", e);
-}
-
 void
 close_fd(struct lsh_fd *fd)
 {
@@ -1841,7 +2138,7 @@ close_fd(struct lsh_fd *fd)
       lsh_oop_cancel_write_fd(fd);
 
       fd->super.alive = 0;
-  
+      /* FIXME: Is this still used? */
       if (fd->fd < 0)
 	/* Unlink the file object, but don't close any
 	 * underlying file. */
@@ -2060,7 +2357,8 @@ lsh_make_pipe(int *fds)
   return 1;
 }
 
-/* Forks a filtering process, and reads the output. */
+/* Forks a filtering process, and reads the output. Always closes
+   the IN fd. */
 int
 lsh_popen(const char *program, const char **argv, int in,
 	  pid_t *child)
@@ -2135,7 +2433,7 @@ lsh_popen_read(const char *program, const char **argv, int in,
     }
 
   if (!s)
-    return s;
+    return NULL;
 
   if (WIFEXITED(status))
     {
@@ -2143,11 +2441,12 @@ lsh_popen_read(const char *program, const char **argv, int in,
 	/* Success. */
 	return s;
 
-      werror("sexp-conv exited with status %i.\n", WEXITSTATUS(status));
+      werror("Program `%z' exited with status %i.\n",
+	     program, WEXITSTATUS(status));
     }
   else
-    werror("sexp-conv terminated by signal %i (%z).\n",
-	   WTERMSIG(status), strsignal(WTERMSIG(status)));
+    werror("Program `%z' terminated by signal %i (%z).\n",
+	   program, WTERMSIG(status), STRSIGNAL(WTERMSIG(status)));
 
   lsh_string_free(s);
   return NULL;
