@@ -39,6 +39,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "nettle/sexp.h"
 
 #include "algorithms.h"
 #include "crypto.h"
@@ -52,8 +53,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include "nettle/sexp.h"
-
 #include "lsh-writekey.c.x"
 
 /* Option parsing */
@@ -210,29 +209,6 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	      
 	      self->label = ssh_format("%lz@%lz", name, host);
 	    }
-	  while (!self->passphrase)
-	    {
-	      struct lsh_string *pw;
-	      struct lsh_string *again;
-
-	      /* FIXME: Move to process_private */
-	      pw = INTERACT_READ_PASSWORD(self->tty, 500,
-					  ssh_format("Enter new passphrase: "), 1);
-	      if (!pw)
-		argp_failure(state, EXIT_FAILURE, 0, "Aborted.");
-
-	      again = INTERACT_READ_PASSWORD(self->tty, 500,
-					     ssh_format("Again: "), 1);
-	      if (!again)
-		argp_failure(state, EXIT_FAILURE, 0, "Aborted.");
-
-	      if (lsh_string_eq(pw, again))
-		self->passphrase = pw;
-	      else
-		lsh_string_free(pw);
-		  
-	      lsh_string_free(again);
-	    }
 	  self->r = (self->server
 		     ? make_system_random()
 		     : make_user_random(getenv(ENV_HOME)));
@@ -357,6 +333,37 @@ process_private(struct lsh_string *key,
                    ALIST_GET(options->crypto_algorithms, ATOM_HMAC_SHA1));
       assert(hmac);
       
+      while (!options->passphrase)
+	{
+	  struct lsh_string *pw;
+	  struct lsh_string *again;
+	  
+	  pw = INTERACT_READ_PASSWORD(options->tty, 500,
+				      ssh_format("Enter new passphrase: "), 1);
+	  if (!pw)
+	    {
+	      werror("Aborted.");
+	      return NULL;
+	    }
+
+	  
+	  again = INTERACT_READ_PASSWORD(options->tty, 500,
+					 ssh_format("Again: "), 1);
+	  if (!again)
+	    {
+	      werror("Aborted.");
+	      lsh_string_free(pw);
+	      return NULL;
+	    }
+
+	  if (lsh_string_eq(pw, again))
+	    options->passphrase = pw;
+	  else
+	    lsh_string_free(pw);
+	  
+	  lsh_string_free(again);
+	}
+
       return spki_pkcs5_encrypt(options->r,
 				options->label,
 				ATOM_HMAC_SHA1,
@@ -399,7 +406,7 @@ main(int argc, char **argv)
   struct lsh_string *input;
   struct lsh_string *output;
   const struct exception *e;
-  
+
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
   if (! (check_file(options->private_file)
@@ -414,6 +421,12 @@ main(int argc, char **argv)
       return EXIT_FAILURE;
     }
   
+  if (!input->length)
+    {
+      werror("Empty key on input, giving up.\n");
+      return EXIT_FAILURE;
+    }
+  
   output = process_private(input, options);
   if (!output)
     return EXIT_FAILURE;
