@@ -31,13 +31,17 @@
  * MA 02111-1307, USA.
  */
 
-#include "twofish.h"
-
-#include "macros.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <assert.h>
 #include <string.h>
 
+#include "twofish.h"
+
+#include "macros.h"
+
 /* Bitwise rotations on 32-bit words.  These are defined as macros that
  * evaluate their argument twice, so do not apply to any expressions with
  * side effects.
@@ -337,7 +341,7 @@ twofish_encrypt(struct twofish_ctx *context,
   uint32_t (*s_box)[256] = context->s_box;
 
   assert( !(length % TWOFISH_BLOCK_SIZE) );
-  for ( ; length; length -= TWOFISH_BLOCK_SIZE);
+  for ( ; length; length -= TWOFISH_BLOCK_SIZE)
     {  
       uint32_t words[4];
       uint32_t r0, r1, r2, r3, t0, t1;
@@ -408,7 +412,7 @@ twofish_decrypt(struct twofish_ctx *context,
   uint32_t (*s_box)[256] = context->s_box;
 
   assert( !(length % TWOFISH_BLOCK_SIZE) );
-  for ( ; length; length -= TWOFISH_BLOCK_SIZE);
+  for ( ; length; length -= TWOFISH_BLOCK_SIZE)
     {  
       uint32_t words[4];
       uint32_t r0, r1, r2, r3, t0, t1;
