@@ -23,14 +23,18 @@
  * MA 02111-1307, USA.
  */
 
-#include "yarrow.h"
-
-#include "macros.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "yarrow.h"
+
+#include "macros.h"
+
 #ifndef YARROW_DEBUG
 #define YARROW_DEBUG 0
 #endif
@@ -162,7 +166,6 @@ yarrow_iterate(uint8_t *digest)
       sha256_update(&hash, sizeof(v0), v0);
       sha256_update(&hash, sizeof(count), count);
 
-      sha256_final(&hash);
       sha256_digest(&hash, SHA256_DIGEST_SIZE, digest);
     }
 }
@@ -191,14 +194,12 @@ yarrow_fast_reseed(struct yarrow256_ctx *ctx)
       sha256_update(&ctx->pools[YARROW_FAST], sizeof(blocks), blocks);
     }
   
-  sha256_final(&ctx->pools[YARROW_FAST]);
   sha256_digest(&ctx->pools[YARROW_FAST], sizeof(digest), digest);
-  sha256_init(&ctx->pools[YARROW_FAST]);
 
   /* Iterate */
   yarrow_iterate(digest);
 
-  aes_set_key(&ctx->key, sizeof(digest), digest);
+  aes_set_encrypt_key(&ctx->key, sizeof(digest), digest);
 
   /* Derive new counter value */
   memset(ctx->counter, 0, sizeof(ctx->counter));
@@ -228,9 +229,7 @@ yarrow_slow_reseed(struct yarrow256_ctx *ctx)
 
   /* Get digest of the slow pool*/
   
-  sha256_final(&ctx->pools[YARROW_SLOW]);
   sha256_digest(&ctx->pools[YARROW_SLOW], sizeof(digest), digest);
-  sha256_init(&ctx->pools[YARROW_SLOW]);
 
   /* Feed it into the fast pool */
   sha256_update(&ctx->pools[YARROW_FAST], sizeof(digest), digest);
@@ -337,7 +336,7 @@ yarrow_gate(struct yarrow256_ctx *ctx)
   for (i = 0; i < sizeof(key); i+= AES_BLOCK_SIZE)
     yarrow_generate_block(ctx, key + i);
 
-  aes_set_key(&ctx->key, sizeof(key), key);
+  aes_set_encrypt_key(&ctx->key, sizeof(key), key);
 }
 
 void
