@@ -1,8 +1,6 @@
 /* publickey_crypto.h
  *
- *
- *
- * $Id: publickey_crypto.h,v 1.43 2000/11/26 23:25:57 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -45,7 +43,7 @@
 */
 
 struct keypair *
-make_keypair(UINT32 type,
+make_keypair(uint32_t type,
 	     struct lsh_string *public,
 	     struct signer *private);
 
@@ -80,7 +78,7 @@ make_keypair(UINT32 type,
        
        ; FIXME: Doesn't handle negative exponents
        (power method void "mpz_t res" "mpz_t g" "mpz_t e")
-       (small_power method void "mpz_t res" "mpz_t g" "UINT32 e"))) */
+       (small_power method void "mpz_t res" "mpz_t g" "uint32_t e"))) */
 
 #define GROUP_RANGE(group, x) ((group)->range((group), (x)))
 #define GROUP_INVERT(group, res, x) ((group)->invert((group), (res), (x)))
@@ -124,7 +122,7 @@ make_ssh_ring_srp_1(void);
      (name dh_method)
      (vars
        (G object abstract_group)
-       (H object hash_algorithm)
+       (H const object hash_algorithm)
        (random object randomness)))
 */
 
@@ -191,7 +189,8 @@ void
 dh_hash_digest(struct dh_instance *self);
 
 struct dh_method *
-make_dh(struct abstract_group *G, struct hash_algorithm *H,
+make_dh(struct abstract_group *G,
+	const struct hash_algorithm *H,
 	struct randomness *r);
 
 struct dh_method *
@@ -202,4 +201,38 @@ init_dh_instance(struct dh_method *m,
 		 struct dh_instance *self,
 		 struct ssh_connection *c);
 
+/* RSA support */
+extern struct signature_algorithm rsa_sha1_algorithm;
+
+/* Non spki keys */
+struct verifier *
+parse_ssh_rsa_public(struct simple_buffer *buffer);
+
+struct verifier *
+make_ssh_rsa_verifier(uint32_t public_length,
+		      const uint8_t *public);
+
+struct lsh_string *
+rsa_generate_key(struct randomness *r, uint32_t bits);
+
+
+/* DSA signatures */
+
+struct signature_algorithm *
+make_dsa_algorithm(struct randomness *random);
+
+/* Non spki keys */
+struct verifier *
+parse_ssh_dss_public(struct simple_buffer *buffer);
+
+struct verifier *
+make_ssh_dss_verifier(uint32_t public_length,
+		      const uint8_t *public);
+
+void dsa_nist_gen(mpz_t p, mpz_t q, struct randomness *r, unsigned l);
+void dsa_find_generator(mpz_t g, struct randomness *r, mpz_t p, mpz_t q);
+
+struct lsh_string *
+dsa_generate_key(struct randomness *r, unsigned level);
+
 #endif /* LSH_PUBLICKEY_CRYPTO_H_INCLUDED */
