@@ -21,6 +21,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <errno.h>
+
+#include <unistd.h>
+#include <fcntl.h>
+
 #include "server.h"
 
 #include "abstract_io.h"
@@ -40,13 +50,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-/* #include <string.h> */
-#include <errno.h>
-
-#include <unistd.h>
-#include <fcntl.h>
-
 #include "server.c.x"
 
 
@@ -195,12 +198,12 @@ read_host_key(const char *file,
   
   int algorithm_name;
 
-    if (fd < 0)
+  if (fd < 0)
     {
       werror("Failed to open `%z' for reading %e\n", file, errno);
       return 0;
     }
-
+  
   contents = io_read_file_raw(fd, 5000);
   if (!contents)
     {
@@ -213,6 +216,8 @@ read_host_key(const char *file,
   s = spki_make_signer(signature_algorithms,
 		       contents,
 		       &algorithm_name);
+  lsh_string_free(contents);
+  
   if (!s)
     {
       werror("Invalid host key\n");
