@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: rl.h,v 1.3 2002/03/23 10:43:56 _cvs_pont Exp $
+ * @(#) $Id: rl.h,v 1.4 2003/05/16 07:34:20 nisse Exp $
  *
  * rl.h
  */
@@ -27,10 +27,6 @@
 #ifndef LSFTP_RL_H
 #define LSFTP_RL_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
