@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: str_utils.h,v 1.2 2002/02/07 12:52:11 nisse Exp $
+ * @(#) $Id: str_utils.h,v 1.4 2003/05/16 07:34:20 nisse Exp $
  *
  * str_utils.h
  */
@@ -27,10 +27,6 @@
 #ifndef LSFTP_STR_UTILS_H
 #define LSFTP_STR_UTILS_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -41,7 +37,7 @@ char* lsftp_unquote( const char* s );
 
 /* Given a string and a set of tokens, place in store (is of size storelen) */
 const char *lsftp_s_strtok(const char *s, const char* tok,
-			   char *store, int storelen); 
+			   char **store); 
 
 const char *lsftp_s_skip(const char *s, const char *sep);
 const char* lsftp_s_skipn(const char *s, const char *sep );
