@@ -1,6 +1,6 @@
 /* tty.h
  *
- * $Id: tty.h,v 1.10 2000/11/15 22:05:09 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -24,25 +24,28 @@
 #ifndef LSH_TTY_H_INCLUDED
 #define LSH_TTY_H_INCLUDED
 
-#include "lsh_types.h"
+#include <inttypes.h>
 
 #include <termios.h>
 
+struct terminal_dimensions
+{
+  uint32_t char_width;
+  uint32_t char_height;
+  uint32_t pixel_width;
+  uint32_t pixel_height;
+};
+
 int tty_getattr(int fd, struct termios *ios);
 int tty_setattr(int fd, struct termios *ios);
 
-#if 0
-int tty_makeraw(int fd);
-#endif
-
-int tty_getwinsize(int fd, UINT32 *w, UINT32 *h, UINT32 *wp, UINT32 *hp);
-int tty_setwinsize(int fd, UINT32 w, UINT32 h, UINT32 wp, UINT32 hp);
-
+int tty_getwinsize(int fd, struct terminal_dimensions *dims);
+int tty_setwinsize(int fd, const struct terminal_dimensions *dims);
 
 struct lsh_string *
 tty_encode_term_mode(struct termios *ios);
 int
-tty_decode_term_mode(struct termios *ios, UINT32 t_len, const UINT8 *t_modes);
+tty_decode_term_mode(struct termios *ios, uint32_t t_len, const uint8_t *t_modes);
 
 #if HAVE_CFMAKERAW
 #define CFMAKERAW cfmakeraw
