@@ -513,10 +513,31 @@ do_read_file(struct lsh_user *u,
 	int fd;
 	close(out[0]);
 
-	if ( (me != user->super.uid) && (seteuid(user->super.uid) < 0) )
+	if (me != user->super.uid)
 	  {
-	    werror("unix_user.c: do_read_file: setuid failed %e\n", errno);
-	    _exit(EXIT_FAILURE);
+	    /* We need to change our persona. We can't change out real
+	     * uid, though, as that might let user processes send
+	     * signals to us. */
+	    if (setgid(user->gid) < 0)
+	      {
+		werror("unix_user.c: do_read_file: setgid failed %e\n", errno);
+		_exit(EXIT_FAILURE);
+	      }
+
+	    /* For simplicity, ignore the user's supplimentary groups.
+	     * They shouldn't be needed to get access to files under
+	     * ~/.lsh. */
+	    if (setgroups(0, NULL) < 0)
+	      {
+		werror("unix_user.c: do_read_file: setgroups failed %e\n", errno);
+		_exit(EXIT_FAILURE);
+	      }
+	      
+	    if (seteuid(user->super.uid) < 0)
+	      {
+		werror("unix_user.c: do_read_file: seteuid failed %e\n", errno);
+		_exit(EXIT_FAILURE);
+	      }
 	  }
 	assert(user->super.uid == geteuid());
 	
@@ -587,7 +608,7 @@ exec_shell(struct unix_user *user, struct spawn_info *info)
 {
   const char **envp;
   const char **argv;
-  const char **shell_argv;
+  const char *program;
   const char *argv0;
 
   char *tz = getenv(ENV_TZ);
@@ -664,35 +685,45 @@ exec_shell(struct unix_user *user, struct spawn_info *info)
 
   debug("exec_shell: argv0 = '%z'.\n", argv0);
 
-  /* Build argument list for lsh-execuv. We need place for
-   *
-   * lsh-execuv -u uid -g gid -n name -i -- $SHELL argv0 <user args> NULL
-   */
-#define MAX_ARG 11
-#define NUMBER(x) lsh_get_cstring(ssh_format("%di", (x)))
-  
-  argv = alloca(sizeof(char *) * (MAX_ARG + info->argc + 1));
   i = 0;
-  argv[i++] = "lsh-execuv";
-  argv[i++] = "-u";
-  trace("exec_shell: After -u\n");
-  argv[i++] = NUMBER(user->super.uid);
-  argv[i++] = "-g";
-  trace("exec_shell: After -g\n");
-  argv[i++] = NUMBER(user->gid);
-  argv[i++] = "-n";
-  argv[i++] = lsh_get_cstring(user->super.name);
-  argv[i++] = "-i";
-  trace("exec_shell: After -i\n");
-  argv[i++] = "--";
-  argv[i++] = lsh_get_cstring(user->shell);
-  shell_argv = argv + i;
-  
-  argv[i++] = argv0;
 
-  assert(i <= MAX_ARG);
+  /* Use lsh-execuv only if we need to change our uid. */
+  if (user->super.uid != getuid())
+    {
+      /* Build argument list for lsh-execuv. We need place for
+       *
+       * lsh-execuv -u uid -g gid -n name -i -- $SHELL argv0 <user args> NULL
+       */
+#define MAX_ARG 10
+#define NUMBER(x) lsh_get_cstring(ssh_format("%di", (x)))
+
+      program = PATH_EXECUV;
+      
+      argv = alloca(sizeof(char *) * (MAX_ARG + info->argc + 2));
+      argv[i++] = "lsh-execuv";
+      argv[i++] = "-u";
+      trace("exec_shell: After -u\n");
+      argv[i++] = NUMBER(user->super.uid);
+      argv[i++] = "-g";
+      trace("exec_shell: After -g\n");
+      argv[i++] = NUMBER(user->gid);
+      argv[i++] = "-n";
+      argv[i++] = lsh_get_cstring(user->super.name);
+      argv[i++] = "-i";
+      trace("exec_shell: After -i\n");
+      argv[i++] = "--";
+      argv[i++] = lsh_get_cstring(user->shell);
+      assert(i <= MAX_ARG);
 #undef MAX_ARG
 #undef NUMBER
+    }
+  else
+    {
+      program = lsh_get_cstring(user->shell);
+      argv = alloca(sizeof(char *) * (info->argc + 2));
+    }
+  argv[i++] = argv0;
+  
   for (j = 0; j<info->argc; j++)
     argv[i++] = info->argv[j];
   argv[i++] = NULL;
@@ -706,13 +737,9 @@ exec_shell(struct unix_user *user, struct spawn_info *info)
   
   trace("exec_shell: before exec\n");
 
-  /* Use lsh-execuv only if we need to change our uid. */
-  if (user->super.uid == getuid())
-    execve(lsh_get_cstring(user->shell), (char **) shell_argv, (char **) envp);
-  else
-    execve(PATH_EXECUV, (char **) argv, (char **) envp);
+  execve(program, (char **) argv, (char **) envp);
 
-  werror("unix_user: exec failed %e\n", errno);
+  werror("unix_user: exec of `%z' failed %e\n", program, errno);
   _exit(EXIT_FAILURE);
 }
 
@@ -1155,3 +1182,13 @@ make_unix_user_db(struct reaper *reaper,
 
   return &self->super;
 }
+
+struct lsh_string * unix_current_user(void)
+{
+  struct passwd *p = getpwuid(getuid());
+
+  if (p)                                   
+    return make_string(p->pw_name);
+  
+  return NULL;
+}
